/// <reference types="node" />
import Element from './Element';
import Task from '@dojo/core/async/Task';
import { Thenable } from '@dojo/shim/interfaces';
import Session from './Session';
import Locator from './lib/Locator';
import { LogEntry, Geolocation, WebDriverCookie } from './interfaces';
/**
 * The Command class is a chainable, subclassable object type that can be used to execute commands serially against a
 * remote WebDriver environment. The standard Command class includes methods from the [[Session]]
 * and [[Element]] classes, so you can perform all standard session and element operations that
 * come with Leadfoot without being forced to author long promise chains.
 *
 * In order to use the Command class, you first need to pass it a [[Session]] instance for it to
 * use:
 *
 * ```js
 * var command = new Command(session);
 * ```
 *
 * Once you have created the Command, you can then start chaining methods, and they will execute in order one after
 * another:
 *
 * ```js
 * command.get('http://example.com')
 *   .findByTagName('h1')
 *   .getVisibleText()
 *   .then(function (text) {
 *       assert.strictEqual(text, 'Example Domain');
 *   });
 * ```
 *
 * Because these operations are asynchronous, you need to use a `then` callback in order to retrieve the value from the
 * last method. Command objects are Thenables, which means that they can be used with any Promises/A+ or ES6-confirmant
 * Promises implementation, though there are some specific differences in the arguments and context that are provided
 * to callbacks; see [[Command.then]] for more details.
 *
 * ---
 *
 * Each call on a Command generates a new Command object, which means that certain operations can be parallelised:
 *
 * ```js
 * command = command.get('http://example.com');
 * Promise.all([
 *   command.getPageTitle(),
 *   command.findByTagName('h1').getVisibleText()
 * ]).then(function (results) {
 *   assert.strictEqual(results[0], results[1]);
 * });
 * ```
 *
 * In this example, the commands on line 3 and 4 both depend upon the `get` call completing successfully but are
 * otherwise independent of each other and so execute here in parallel. This is different from commands in Intern 1
 * which were always chained onto the last called method within a given test.
 *
 * ---
 *
 * Command objects actually encapsulate two different types of interaction: *session* interactions, which operate
 * against the entire browser session, and *element* interactions, which operate against specific elements taken from
 * the currently loaded page. Things like navigating the browser, moving the mouse cursor, and executing scripts are
 * session interactions; things like getting text displayed on the page, typing into form fields, and getting element
 * attributes are element interactions.
 *
 * Session interactions can be performed at any time, from any Command. On the other hand, to perform element
 * interactions, you first need to retrieve one or more elements to interact with. This can be done using any of the
 * `find` or `findAll` methods, by the `getActiveElement` method, or by returning elements from `execute` or
 * `executeAsync` calls. The retrieved elements are stored internally as the *element context* of all chained
 * Commands. When an element method is called on a chained Command with a single element context, the result will be
 * returned as-is:
 *
 * ```js
 * command = command.get('http://example.com')
 *   // finds one element -> single element context
 *   .findByTagName('h1')
 *   .getVisibleText()
 *   .then(function (text) {
 *     // `text` is the text from the element context
 *     assert.strictEqual(text, 'Example Domain');
 *   });
 * ```
 *
 * When an element method is called on a chained Command with a multiple element context, the result will be returned
 * as an array:
 *
 * ```js
 * command = command.get('http://example.com')
 *   // finds multiple elements -> multiple element context
 *   .findAllByTagName('p')
 *   .getVisibleText()
 *   .then(function (texts) {
 *     // `texts` is an array of text from each of the `p` elements
 *     assert.deepEqual(texts, [
 *       'This domain is established to be used for […]',
 *       'More information...'
 *     ]);
 *   });
 * ```
 *
 * The `find` and `findAll` methods are special and change their behaviour based on the current element filtering state
 * of a given command. If a command has been filtered by element, the `find` and `findAll` commands will only find
 * elements *within* the currently filtered set of elements. Otherwise, they will find elements throughout the page.
 *
 * Some method names, like `click`, are identical for both Session and Element APIs; in this case, the element APIs
 * are suffixed with the word `Element` in order to identify them uniquely.
 *
 * ---
 *
 * Commands can be subclassed in order to add additional functionality without making direct modifications to the
 * default Command prototype that might break other parts of the system:
 *
 * ```js
 * function CustomCommand() {
 *   Command.apply(this, arguments);
 * }
 * CustomCommand.prototype = Object.create(Command.prototype);
 * CustomCommand.prototype.constructor = CustomCommand;
 * CustomCommand.prototype.login = function (username, password) {
 *   return new this.constructor(this, function () {
 *     return this.parent
 *       .findById('username')
 *         .click()
 *         .type(username)
 *         .end()
 *       .findById('password')
 *         .click()
 *         .type(password)
 *         .end()
 *       .findById('login')
 *         .click()
 *         .end();
 *   });
 * };
 * ```
 *
 * Note that returning `this`, or a command chain starting from `this`, from a callback or command initialiser will
 * deadlock the Command, as it waits for itself to settle before settling.
 */
export default class Command<T> extends Locator<Command<Element>, Command<Element[]>, Command<void>> {
    /**
     * Augments `target` with a conversion of the `originalFn` method that enables its use with a Command object.
     * This can be used to easily add new methods from any custom object that implements the Session API to any target
     * object that implements the Command API.
     *
     * Functions that are copied may have the following extra properties in order to change the way that Command works
     * with these functions:
     *
     * - `createsContext` (boolean): If this property is specified, the return value from the function will be used as
     *   the new context for the returned Command.
     * - `usesElement` (boolean): If this property is specified, element(s) from the current context will be used as
     *   the first argument to the function, if the explicitly specified first argument is not already an element.
     *
     * @memberOf module:leadfoot/Command
     * @param {module:leadfoot/Command} target
     * @param {string} key
     * @param {Function} originalFn
     */
    static addSessionMethod<U>(target: Command<U>, key: string, originalFn: Function): void;
    /**
     * Augments `target` with a method that will call `key` on all context elements stored within `target`.
     * This can be used to easily add new methods from any custom object that implements the Element API to any target
     * object that implements the Command API.
     *
     * Functions that are copied may have the following extra properties in order to change the way that Command works
     * with these functions:
     *
     * - `createsContext` (boolean): If this property is specified, the return value from the function will be used as
     *   the new context for the returned Command.
     *
     * @memberOf module:leadfoot/Command
     * @param {module:leadfoot/Command} target
     * @param {string} key
     */
    static addElementMethod<T>(target: Command<T>, key: string): void;
    private _parent;
    private _session;
    private _context;
    private _task;
    /**
     * @param parent
     * The parent command that this command is chained to, or a [[Sesssion]] object if this is the
     * first command in a command chain.
     *
     * @param initialiser
     * A function that will be executed when all parent commands have completed execution. This function can create a
     * new context for this command by calling the passed `setContext` function any time prior to resolving the Promise
     * that it returns. If no context is explicitly provided, the context from the parent command will be used.
     *
     * @param errback
     * A function that will be executed if any parent commands failed to complete successfully. This function can create
     * a new context for the current command by calling the passed `setContext` function any time prior to resolving the
     * Promise that it returns. If no context is explicitly provided, the context from the parent command will be used.
     */
    constructor(parent: Session | Command<any>, initialiser?: (setContext: Function, value: any) => any | Task<any>, errback?: (setContext: Function, error: Error) => any | Task<any>);
    /**
     * The parent Command of the Command, if one exists.
     *
     * @readonly
     */
    readonly parent: Session | Command<any>;
    /**
     * The parent Session of the Command.
     *
     * @readonly
     */
    readonly session: Session;
    /**
     * The filtered elements that will be used if an element-specific method is invoked. Note that this property is not
     * valid until the parent Command has been settled. The context array also has two additional properties:
     *
     * - isSingle (boolean): If true, the context will always contain a single element. This is used to differentiate
     *   between methods that should still return scalar values (`find`) and methods that should return arrays of
     *   values even if there is only one element in the context (`findAll`).
     * - depth (number): The depth of the context within the command chain. This is used to prevent traversal into
     *   higher filtering levels by [[Command.end]].
     *
     * @readonly
     */
    readonly context: Context;
    /**
     * The underlying Promise for the Command.
     *
     * @readonly
     */
    readonly promise: Task<any>;
    /**
     * Pauses execution of the next command in the chain for `ms` milliseconds.
     *
     * @param ms Time to delay, in milliseconds.
     */
    sleep(ms: number): Command<{}>;
    /**
     * Ends the most recent filtering operation in the current Command chain and returns the set of matched elements
     * to the previous state. This is equivalent to the `jQuery#end` method.
     *
     * @example
     * ```js
     * command
     *   .findById('parent') // sets filter to #parent
     *     .findByClassName('child') // sets filter to all .child inside #parent
     *       .getVisibleText()
     *       .then(function (visibleTexts) {
     *         // all the visible texts from the children
     *       })
     *       .end() // resets filter to #parent
     *     .end(); // resets filter to nothing (the whole document)
     *  ```
     *
     * @param numCommandsToPop The number of element contexts to pop. Defaults to 1.
     */
    end(numCommandsToPop?: number): Command<void>;
    /**
     * Adds a callback to be invoked once the previously chained operation has completed.
     *
     * This method is compatible with the `Promise#then` API, with two important differences:
     *
     * 1. The context (`this`) of the callback is set to the Command object, rather than being `undefined`. This allows
     *    promise helpers to be created that can retrieve the appropriate session and element contexts for execution.
     * 2. A second non-standard `setContext` argument is passed to the callback. This `setContext` function can be
     *    called at any time before the callback fulfills its return value and expects either a single
     *    [[Element]] or an array of Elements to be provided as its only argument. The provided
     *    element(s) will be used as the context for subsequent element method invocations (`click`, etc.). If
     *    the `setContext` method is not called, the element context from the parent will be passed through unmodified.
     */
    then<U>(callback?: (value: T, setContext: SetContextMethod<U>) => U | Thenable<U> | Command<U>, errback?: (error: Error) => void | U | Thenable<U> | Command<U>): Command<{}>;
    /**
     * Adds a callback to be invoked when any of the previously chained operations have failed.
     */
    catch(errback: (reason: Error) => void | T | Command<T>): Command<{}>;
    /**
     * Adds a callback to be invoked once the previously chained operations have resolved.
     */
    finally(callback: () => void): this;
    /**
     * Cancels all outstanding chained operations of the Command. Calling this method will cause this command and all
     * subsequent chained commands to fail with a CancelError.
     */
    cancel(): this;
    find(strategy: string, value: string): Command<Element>;
    findAll(strategy: string, value: string): Command<Element[]>;
    findDisplayed(strategy: string, value: string): Command<Element>;
    /**
     * a function that, when called, creates a new Command that retrieves elements from the parent context and
     * uses them as the context for the newly created Command.
     */
    private _callFindElementMethod<U>(key, ...args);
    private _callElementMethod<U>(key, ...args);
    private _callSessionMethod<U>(key, ...args);
    /**
     * Gets the current value of a timeout for the session.
     *
     * @param type The type of timeout to retrieve. One of 'script', 'implicit', or 'page load'.
     * @returns The timeout, in milliseconds.
     */
    getTimeout(type: string): Command<number>;
    /**
     * Sets the value of a timeout for the session.
     *
     * @param type
     * The type of timeout to set. One of 'script', 'implicit', or 'page load'.
     *
     * @param ms
     * The length of time to use for the timeout, in milliseconds. A value of 0 will cause operations to time out
     * immediately.
     */
    setTimeout(type: string, ms: number): Command<void>;
    /**
     * Gets the identifier for the window that is currently focused.
     *
     * @returns A window handle identifier that can be used with other window handling functions.
     */
    getCurrentWindowHandle(): Command<string>;
    /**
     * Gets a list of identifiers for all currently open windows.
     */
    getAllWindowHandles(): Command<string[]>;
    /**
     * Gets the URL that is loaded in the focused window/frame.
     */
    getCurrentUrl(): Command<string>;
    /**
     * Navigates the focused window/frame to a new URL.
     */
    get(url: string): Command<void>;
    /**
     * Navigates the focused window/frame forward one page using the browser’s navigation history.
     */
    goForward(): Command<void>;
    /**
     * Navigates the focused window/frame back one page using the browser’s navigation history.
     */
    goBack(): Command<void>;
    /**
     * Reloads the current browser window/frame.
     */
    refresh(): Command<void>;
    /**
     * Executes JavaScript code within the focused window/frame. The code should return a value synchronously.
     *
     * @see [[Command.executeAsync]] to execute code that returns values asynchronously.
     *
     * @param script
     * The code to execute. This function will always be converted to a string, sent to the remote environment, and
     * reassembled as a new anonymous function on the remote end. This means that you cannot access any variables
     * through closure. If your code needs to get data from variables on the local end, they should be passed using
     * `args`.
     *
     * @param args
     * An array of arguments that will be passed to the executed code. Only values that can be serialised to JSON, plus
     * [[Element]] objects, can be specified as arguments.
     *
     * @returns
     * The value returned by the remote code. Only values that can be serialised to JSON, plus DOM elements, can be
     * returned.
     */
    execute(script: Function | string, args?: any[]): Command<any>;
    /**
     * Executes JavaScript code within the focused window/frame. The code must invoke the provided callback in
     * order to signal that it has completed execution.
     *
     * @see [[Command.execute]] to execute code that returns values synchronously.
     * @see [[Command.setExecuteAsyncTimeout]] to set the time until an asynchronous script is
     * considered timed out.
     *
     * @param script
     * The code to execute. This function will always be converted to a string, sent to the remote environment, and
     * reassembled as a new anonymous function on the remote end. This means that you cannot access any variables
     * through closure. If your code needs to get data from variables on the local end, they should be passed using
     * `args`.
     *
     * @param args
     * An array of arguments that will be passed to the executed code. Only values that can be serialised to JSON, plus
     * [[Element]] objects, can be specified as arguments. In addition to these arguments, a
     * callback function will always be passed as the final argument to the function specified in `script`. This
     * callback function must be invoked in order to signal that execution has completed. The return value of the
     * execution, if any, should be passed to this callback function.
     *
     * @returns
     * The value returned by the remote code. Only values that can be serialised to JSON, plus DOM elements, can be
     * returned.
     */
    executeAsync(script: Function | string, args?: any[]): Command<any>;
    /**
     * Gets a screenshot of the focused window and returns it in PNG format.
     */
    takeScreenshot(): Command<Buffer>;
    /**
     * Gets a list of input method editor engines available to the remote environment.
     * As of April 2014, no known remote environments support IME functions.
     */
    getAvailableImeEngines(): Command<string[]>;
    /**
     * Gets the currently active input method editor for the remote environment.
     * As of April 2014, no known remote environments support IME functions.
     */
    getActiveImeEngine(): Command<string>;
    /**
     * Returns whether or not an input method editor is currently active in the remote environment.
     * As of April 2014, no known remote environments support IME functions.
     */
    isImeActivated(): Command<boolean>;
    /**
     * Deactivates any active input method editor in the remote environment.
     * As of April 2014, no known remote environments support IME functions.
     */
    deactivateIme(): Command<void>;
    /**
     * Activates an input method editor in the remote environment.
     * As of April 2014, no known remote environments support IME functions.
     *
     * @param engine The type of IME to activate.
     */
    activateIme(engine: string): Command<void>;
    /**
     * Switches the currently focused frame to a new frame.
     *
     * @param id
     * The frame to switch to. In most environments, a number or string value corresponds to a key in the
     * `window.frames` object of the currently active frame. If `null`, the topmost (default) frame will be used.
     * If an Element is provided, it must correspond to a `<frame>` or `<iframe>` element.
     */
    switchToFrame(id: string | number | Element): Command<void>;
    /**
     * Switches the currently focused window to a new window.
     *
     * @param handle
     * The handle of the window to switch to. In mobile environments and environments based on the W3C WebDriver
     * standard, this should be a handle as returned by [[Command.getAllWindowHandles]].
     *
     * In environments using the JsonWireProtocol, this value corresponds to the `window.name` property of a window.
     */
    switchToWindow(handle: string): Command<void>;
    /**
     * Switches the currently focused frame to the parent of the currently focused frame.
     */
    switchToParentFrame(): Command<void>;
    /**
     * Closes the currently focused window. In most environments, after the window has been closed, it is necessary
     * to explicitly switch to whatever window is now focused.
     */
    closeCurrentWindow(): Command<void>;
    /**
     * Sets the dimensions of a window.
     *
     * @param windowHandle
     * The name of the window to resize. See [[Command.switchToWindow]] to learn about valid
     * window names. Omit this argument to resize the currently focused window.
     *
     * @param width
     * The new width of the window, in CSS pixels.
     *
     * @param height
     * The new height of the window, in CSS pixels.
     */
    setWindowSize(width: number, height: number): Command<void>;
    setWindowSize(windowHandle: string, width: number, height: number): Command<void>;
    /**
     * Gets the dimensions of a window.
     *
     * @param windowHandle
     * The name of the window to query. See [[Command.switchToWindow]] to learn about valid
     * window names. Omit this argument to query the currently focused window.
     *
     * @returns
     * An object describing the width and height of the window, in CSS pixels.
     */
    getWindowSize(windowHandle?: string): Command<{
        width: number;
        height: number;
    }>;
    /**
     * Sets the position of a window.
     *
     * Note that this method is not part of the W3C WebDriver standard.
     *
     * @param windowHandle
     * The name of the window to move. See [[Command.switchToWindow]] to learn about valid
     * window names. Omit this argument to move the currently focused window.
     *
     * @param x
     * The screen x-coordinate to move to, in CSS pixels, relative to the left edge of the primary monitor.
     *
     * @param y
     * The screen y-coordinate to move to, in CSS pixels, relative to the top edge of the primary monitor.
     */
    setWindowPosition(x: number, y: number): Command<void>;
    setWindowPosition(windowHandle: string, x: number, y: number): Command<void>;
    /**
     * Gets the position of a window.
     *
     * Note that this method is not part of the W3C WebDriver standard.
     *
     * @param windowHandle
     * The name of the window to query. See [[Command.switchToWindow]] to learn about valid
     * window names. Omit this argument to query the currently focused window.
     *
     * @returns
     * An object describing the position of the window, in CSS pixels, relative to the top-left corner of the
     * primary monitor. If a secondary monitor exists above or to the left of the primary monitor, these values
     * will be negative.
     */
    getWindowPosition(windowHandle?: string): Command<{
        x: number;
        y: number;
    }>;
    /**
     * Maximises a window according to the platform’s window system behaviour.
     *
     * @param windowHandle
     * The name of the window to resize. See [[Command.switchToWindow] to learn about valid
     * window names. Omit this argument to resize the currently focused window.
     */
    maximizeWindow(windowHandle?: string): Command<void>;
    /**
     * Gets all cookies set on the current page.
     */
    getCookies(): Command<WebDriverCookie[]>;
    /**
     * Sets a cookie on the current page.
     */
    setCookie(cookie: WebDriverCookie): Command<void>;
    /**
     * Clears all cookies for the current page.
     */
    clearCookies(): Command<void>;
    /**
     * Deletes a cookie on the current page.
     *
     * @param name The name of the cookie to delete.
     */
    deleteCookie(name: string): Command<void>;
    /**
     * Gets the HTML loaded in the focused window/frame. This markup is serialised by the remote environment so
     * may not exactly match the HTML provided by the Web server.
     */
    getPageSource(): Command<string>;
    /**
     * Gets the title of the top-level browsing context of the current window or tab.
     */
    getPageTitle(): Command<string>;
    /**
     * Gets the currently focused element from the focused window/frame.
     */
    getActiveElement(): Command<Element>;
    /**
     * Types into the focused window/frame/element.
     *
     * @param keys
     * The text to type in the remote environment. It is possible to type keys that do not have normal character
     * representations (modifier keys, function keys, etc.) as well as keys that have two different representations
     * on a typical US-ASCII keyboard (numpad keys); use the values from [[keys]] to type these
     * special characters. Any modifier keys that are activated by this call will persist until they are
     * deactivated. To deactivate a modifier key, type the same modifier key a second time, or send `\uE000`
     * ('NULL') to deactivate all currently active modifier keys.
     */
    pressKeys(keys: string | string[]): Command<void>;
    /**
     * Gets the current screen orientation.
     */
    getOrientation(): Command<"portrait" | "landscape">;
    /**
     * Sets the screen orientation.
     *
     * @param orientation Either 'portrait' or 'landscape'.
     */
    setOrientation(orientation: 'portrait' | 'landscape'): Command<void>;
    /**
     * Gets the text displayed in the currently active alert pop-up.
     */
    getAlertText(): Command<string>;
    /**
     * Types into the currently active prompt pop-up.
     *
     * @param text The text to type into the pop-up’s input box.
     */
    typeInPrompt(text: string | string[]): Command<void>;
    /**
     * Accepts an alert, prompt, or confirmation pop-up. Equivalent to clicking the 'OK' button.
     */
    acceptAlert(): Command<void>;
    /**
     * Dismisses an alert, prompt, or confirmation pop-up. Equivalent to clicking the 'OK' button of an alert pop-up
     * or the 'Cancel' button of a prompt or confirmation pop-up.
     */
    dismissAlert(): Command<void>;
    /**
     * Moves the remote environment’s mouse cursor to the specified element or relative position. If the element is
     * outside of the viewport, the remote driver will attempt to scroll it into view automatically.
     *
     * @param element
     * The element to move the mouse to. If x-offset and y-offset are not specified, the mouse will be moved to the
     * centre of the element.
     *
     * @param xOffset
     * The x-offset of the cursor, maybe in CSS pixels, relative to the left edge of the specified element’s
     * bounding client rectangle. If no element is specified, the offset is relative to the previous position of the
     * mouse, or to the left edge of the page’s root element if the mouse was never moved before.
     *
     * @param yOffset
     * The y-offset of the cursor, maybe in CSS pixels, relative to the top edge of the specified element’s bounding
     * client rectangle. If no element is specified, the offset is relative to the previous position of the mouse,
     * or to the top edge of the page’s root element if the mouse was never moved before.
     */
    moveMouseTo(xOffset?: number, yOffset?: number): Command<void>;
    moveMouseTo(element?: Element, xOffset?: number, yOffset?: number): Command<void>;
    /**
     * Clicks a mouse button at the point where the mouse cursor is currently positioned. This method may fail to
     * execute with an error if the mouse has not been moved anywhere since the page was loaded.
     *
     * @param button
     * The button to click. 0 corresponds to the primary mouse button, 1 to the middle mouse button, 2 to the
     * secondary mouse button. Numbers above 2 correspond to any additional buttons a mouse might provide.
     */
    clickMouseButton(button?: number): Command<void>;
    /**
     * Depresses a mouse button without releasing it.
     *
     * @param button The button to press. See [[Command.click]] for available options.
     */
    pressMouseButton(button?: number): Command<void>;
    /**
     * Releases a previously depressed mouse button.
     *
     * @param button The button to press. See [[Command.click]] for available options.
     */
    releaseMouseButton(button?: number): Command<void>;
    /**
     * Double-clicks the primary mouse button.
     */
    doubleClick(): Command<void>;
    /**
     * Taps an element on a touch screen device. If the element is outside of the viewport, the remote driver will
     * attempt to scroll it into view automatically.
     *
     * @param element The element to tap.
     */
    tap(element: Element): Command<void>;
    /**
     * Depresses a new finger at the given point on a touch screen device without releasing it.
     *
     * @param x The screen x-coordinate to press, maybe in device pixels.
     * @param y The screen y-coordinate to press, maybe in device pixels.
     */
    pressFinger(x: number, y: number): Command<void>;
    /**
     * Releases whatever finger exists at the given point on a touch screen device.
     *
     * @param x The screen x-coordinate where a finger is pressed, maybe in device pixels.
     * @param y The screen y-coordinate where a finger is pressed, maybe in device pixels.
     */
    releaseFinger(x: number, y: number): Command<void>;
    /**
     * Moves the last depressed finger to a new point on the touch screen.
     *
     * @param x The screen x-coordinate to move to, maybe in device pixels.
     * @param y The screen y-coordinate to move to, maybe in device pixels.
     */
    moveFinger(x: number, y: number): Command<void>;
    /**
     * Scrolls the currently focused window on a touch screen device.
     *
     * @param element
     * An element to scroll to. The window will be scrolled so the element is as close to the top-left corner of the
     * window as possible.
     *
     * @param xOffset
     * An optional x-offset, relative to the left edge of the element, in CSS pixels. If no element is specified,
     * the offset is relative to the previous scroll position of the window.
     *
     * @param yOffset
     * An optional y-offset, relative to the top edge of the element, in CSS pixels. If no element is specified,
     * the offset is relative to the previous scroll position of the window.
     */
    touchScroll(xOffset: number, yOffset: number): Command<void>;
    touchScroll(element?: Element, xOffset?: number, yOffset?: number): Command<void>;
    /**
     * Performs a double-tap gesture on an element.
     *
     * @method
     * @param element The element to double-tap.
     */
    doubleTap(element?: Element): Command<void>;
    /**
     * Performs a long-tap gesture on an element.
     *
     * @method
     * @param element The element to long-tap.
     */
    longTap(element?: Element): Command<void>;
    /**
     * Flicks a finger. Note that this method is currently badly specified and highly dysfunctional and is only
     * provided for the sake of completeness.
     *
     * @param element The element where the flick should start.
     * @param xOffset The x-offset in pixels to flick by.
     * @param yOffset The x-offset in pixels to flick by.
     * @param speed The speed of the flick, in pixels per *second*. Most human flicks are 100–200ms, so
     * this value will be higher than expected.
     */
    flickFinger(element: Element, xOffset: number, yOffset: number, speed?: number): Command<void>;
    flickFinger(xOffset: number, yOffset: number, speed?: number): Command<void>;
    /**
     * Gets the current geographical location of the remote environment.
     *
     * @returns
     * Latitude and longitude are specified using standard WGS84 decimal latitude/longitude. Altitude is specified
     * as meters above the WGS84 ellipsoid. Not all environments support altitude.
     */
    getGeolocation(): Command<Geolocation>;
    /**
     * Sets the geographical location of the remote environment.
     *
     * @param location
     * Latitude and longitude are specified using standard WGS84 decimal latitude/longitude. Altitude is specified
     * as meters above the WGS84 ellipsoid. Not all environments support altitude.
     */
    setGeolocation(location: Geolocation): Command<void>;
    /**
     * Gets all logs from the remote environment of the given type. The logs in the remote environment are cleared
     * once they have been retrieved.
     *
     * @param type
     * The type of log entries to retrieve. Available log types differ between remote environments. Use
     * [[Command.getAvailableLogTypes]] to learn what log types are currently available. Not all
     * environments support all possible log types.
     *
     * @returns
     * An array of log entry objects. Timestamps in log entries are Unix timestamps, in seconds.
     */
    getLogsFor(type: string): Command<LogEntry[]>;
    /**
     * Gets the types of logs that are currently available for retrieval from the remote environment.
     */
    getAvailableLogTypes(): Command<string[]>;
    /**
     * Gets the current state of the HTML5 application cache for the current page.
     *
     * @returns
     * The cache status. One of 0 (uncached), 1 (cached/idle), 2 (checking), 3 (downloading), 4 (update ready), 5
     * (obsolete).
     */
    getApplicationCacheStatus(): Command<number>;
    /**
     * Terminates the session. No more commands will be accepted by the remote after this point.
     */
    quit(): Command<void>;
    /**
     * Waits for all elements in the currently active window/frame to be destroyed.
     *
     * @param using
     * The element retrieval strategy to use. See [[Command.find]] for options.
     *
     * @param value
     * The strategy-specific value to search for. See [[Command.find]] for details.
     */
    waitForDeleted(using: string, value: string): Command<void>;
    /**
     * Gets the timeout for [[Command.executeAsync]] calls.
     */
    getExecuteAsyncTimeout(): Command<number>;
    /**
     * Sets the timeout for [[Command.executeAsync]] calls.
     *
     * @param ms The length of the timeout, in milliseconds.
     */
    setExecuteAsyncTimeout(ms: number): Command<void>;
    /**
     * Gets the timeout for [[Command.find]] calls.
     */
    getFindTimeout(): Command<number>;
    /**
     * Sets the timeout for [[Command.find]] calls.
     *
     * @param ms The length of the timeout, in milliseconds.
     */
    setFindTimeout(ms: number): Command<void>;
    /**
     * Gets the timeout for [[Command.get]] calls.
     */
    getPageLoadTimeout(): Command<number>;
    /**
     * Sets the timeout for [[Command.get]] calls.
     *
     * @param ms The length of the timeout, in milliseconds.
     */
    setPageLoadTimeout(ms: string): Command<void>;
    /**
     * Clicks the element. This method works on both mouse and touch platforms.
     */
    click(): Command<void>;
    /**
     * Submits the element, if it is a form, or the form belonging to the element, if it is a form element.
     */
    submit(): Command<void>;
    /**
     * Gets the visible text within the element. `<br>` elements are converted to line breaks in the returned
     * text, and whitespace is normalised per the usual XML/HTML whitespace normalisation rules.
     */
    getVisibleText(): Command<string>;
    /**
     * Types into the element. This method works the same as the [[Command.pressKeys]] method
     * except that any modifier keys are automatically released at the end of the command. This method should be used
     * instead of [[Command.pressKeys]] to type filenames into file upload fields.
     *
     * Since 1.5, if the WebDriver server supports remote file uploads, and you type a path to a file on your local
     * computer, that file will be transparently uploaded to the remote server and the remote filename will be typed
     * instead. If you do not want to upload local files, use [[Command.pressKeys]] instead.
     *
     * @param value
     * The text to type in the remote environment. See [[Command.pressKeys]] for more information.
     */
    type(value: string | string[]): Command<void>;
    /**
     * Gets the tag name of the element. For HTML documents, the value is always lowercase.
     */
    getTagName(): Command<string>;
    /**
     * Clears the value of a form element.
     */
    clearValue(): Command<void>;
    /**
     * Returns whether or not a form element is currently selected (for drop-down options and radio buttons), or
     * whether or not the element is currently checked (for checkboxes).
     */
    isSelected(): Command<boolean>;
    /**
     * Returns whether or not a form element can be interacted with.
     */
    isEnabled(): Command<boolean>;
    /**
     * Gets a property or attribute of the element according to the WebDriver specification algorithm. Use of this
     * method is not recommended; instead, use [[Command.getAttribute]] to retrieve DOM attributes
     * and [[Command.getProperty]] to retrieve DOM properties.
     *
     * This method uses the following algorithm on the server to determine what value to return:
     *
     * 1. If `name` is 'style', returns the `style.cssText` property of the element.
     * 2. If the attribute exists and is a boolean attribute, returns 'true' if the attribute is true, or null
     *    otherwise.
     * 3. If the element is an `<option>` element and `name` is 'value', returns the `value` attribute if it exists,
     *    otherwise returns the visible text content of the option.
     * 4. If the element is a checkbox or radio button and `name` is 'selected', returns 'true' if the element is
     *    checked, or null otherwise.
     * 5. If the returned value is expected to be a URL (e.g. element is `<a>` and attribute is `href`), returns the
     *    fully resolved URL from the `href`/`src` property of the element, not the attribute.
     * 6. If `name` is 'class', returns the `className` property of the element.
     * 7. If `name` is 'readonly', returns 'true' if the `readOnly` property is true, or null otherwise.
     * 8. If `name` corresponds to a property of the element, and the property is not an Object, return the property
     *    value coerced to a string.
     * 9. If `name` corresponds to an attribute of the element, return the attribute value.
     *
     * @param name The property or attribute name.
     * @returns The value of the attribute as a string, or `null` if no such property or
     * attribute exists.
     */
    getSpecAttribute(name: string): Command<string>;
    /**
     * Gets an attribute of the element.
     *
     * @see Element#getProperty to retrieve an element property.
     * @param name The name of the attribute.
     * @returns The value of the attribute, or `null` if no such attribute exists.
     */
    getAttribute(name: string): Command<{}>;
    /**
     * Gets a property of the element.
     *
     * @see Element#getAttribute to retrieve an element attribute.
     * @param name The name of the property.
     * @returns The value of the property.
     */
    getProperty(name: string): Command<{}>;
    /**
     * Determines if this element is equal to another element.
     */
    equals(other: Element): Command<boolean>;
    /**
     * Returns whether or not the element would be visible to an actual user. This means that the following types
     * of elements are considered to be not displayed:
     *
     * 1. Elements with `display: none`
     * 2. Elements with `visibility: hidden`
     * 3. Elements positioned outside of the viewport that cannot be scrolled into view
     * 4. Elements with `opacity: 0`
     * 5. Elements with no `offsetWidth` or `offsetHeight`
     */
    isDisplayed(): Command<boolean>;
    /**
     * Gets the position of the element relative to the top-left corner of the document, taking into account
     * scrolling and CSS transformations (if they are supported).
     */
    getPosition(): Command<{
        x: number;
        y: number;
    }>;
    /**
     * Gets the size of the element, taking into account CSS transformations (if they are supported).
     */
    getSize(): Command<{
        width: number;
        height: number;
    }>;
    /**
     * Gets a CSS computed property value for the element.
     *
     * @param propertyName
     * The CSS property to retrieve. This argument must be hyphenated, *not* camel-case.
     */
    getComputedStyle(propertyName: string): Command<string>;
}
export interface SetContextMethod<T> {
    (context: T | T[]): void;
}
export interface Context extends Array<any> {
    isSingle?: boolean;
    depth?: number;
}
