import Locator from './lib/Locator';
import Task from '@dojo/core/async/Task';
import Session from './Session';
/**
 * An Element represents a DOM or UI element within the remote environment.
 */
export default class Element extends Locator<Task<Element>, Task<Element[]>, Task<void>> {
    private _elementId;
    private _session;
    /**
     * @constructor module:leadfoot/Element
     *
     * @param elementId
     * The ID of the element, as provided by the remote.
     *
     * @param session
     * The session that the element belongs to.
     */
    constructor(elementId: any, session?: Session);
    /**
     * The opaque, remote-provided ID of the element.
     *
     * @member elementId
     * @readonly
     */
    readonly elementId: string;
    /**
     * The [[Session]] that the element belongs to.
     * @readonly
     */
    readonly session: Session;
    private _get<T>(path, requestData?, pathParts?);
    private _post<T>(path, requestData?, pathParts?);
    toJSON(): {
        ELEMENT: string;
    };
    /**
     * Normalize whitespace in the same way that most browsers generate innerText.
     *
     * @param text
     * @returns Text with leading and trailing whitespace removed, with inner runs of spaces changed to a
     * single space, and with "\r\n" pairs converted to "\n".
     */
    private _normalizeWhitespace(text);
    /**
     * Uploads a file to a remote Selenium server for use when testing file uploads. This API is not part of the
     * WebDriver specification and should not be used directly. To send a file to a server that supports file uploads,
     * use [[Element.type]] to type the name of the local file into a file input field and the file
     * will be transparently transmitted and used by the server.
     */
    private _uploadFile(filename);
    /**
     * Gets the first element within this element that matches the given query.
     *
     * @see [[Session.setFindTimeout]] to set the amount of time it the remote environment
     * should spend waiting for an element that does not exist at the time of the `find` call before timing
     * out.
     *
     * @param using
     * The element retrieval strategy to use. See [[Session.find]] for options.
     *
     * @param value
     * The strategy-specific value to search for. See [[Session.find]] for details.
     */
    find(using: string, value: string): Task<Element>;
    /**
     * Gets all elements within this element that match the given query.
     *
     * @param using
     * The element retrieval strategy to use. See [[Session.find]] for options.
     *
     * @param value
     * The strategy-specific value to search for. See [[Session.find]] for details.
     */
    findAll(using: string, value: string): Task<Element[]>;
    /**
     * Clicks the element. This method works on both mouse and touch platforms.
     */
    click(): Task<void>;
    /**
     * Submits the element, if it is a form, or the form belonging to the element, if it is a form element.
     */
    submit(): Task<any>;
    /**
     * Gets the visible text within the element. `<br>` elements are converted to line breaks in the returned
     * text, and whitespace is normalised per the usual XML/HTML whitespace normalisation rules.
     */
    getVisibleText(): Task<string>;
    /**
     * Types into the element. This method works the same as the [[Session.pressKeys]] method
     * except that any modifier keys are automatically released at the end of the command. This method should be used
     * instead of [[Session.pressKeys]] to type filenames into file upload fields.
     *
     * Since 1.5, if the WebDriver server supports remote file uploads, and you type a path to a file on your local
     * computer, that file will be transparently uploaded to the remote server and the remote filename will be typed
     * instead. If you do not want to upload local files, use [[Session.pressKeys]] instead.
     *
     * @param value
     * The text to type in the remote environment. See [[Session.pressKeys]] for more information.
     */
    type(value: string | string[]): Task<void>;
    /**
     * Gets the tag name of the element. For HTML documents, the value is always lowercase.
     */
    getTagName(): Task<string>;
    /**
     * Clears the value of a form element.
     */
    clearValue(): Task<void>;
    /**
     * Returns whether or not a form element is currently selected (for drop-down options and radio buttons), or
     * whether or not the element is currently checked (for checkboxes).
     */
    isSelected(): Task<boolean>;
    /**
     * Returns whether or not a form element can be interacted with.
     */
    isEnabled(): Task<boolean>;
    /**
     * Gets a property or attribute of the element according to the WebDriver specification algorithm. Use of this
     * method is not recommended; instead, use [[Element.getAttribute]] to retrieve DOM attributes
     * and [[Element.getProperty]] to retrieve DOM properties.
     *
     * This method uses the following algorithm on the server to determine what value to return:
     *
     * 1. If `name` is 'style', returns the `style.cssText` property of the element.
     * 2. If the attribute exists and is a boolean attribute, returns 'true' if the attribute is true, or null
     *    otherwise.
     * 3. If the element is an `<option>` element and `name` is 'value', returns the `value` attribute if it exists,
     *    otherwise returns the visible text content of the option.
     * 4. If the element is a checkbox or radio button and `name` is 'selected', returns 'true' if the element is
     *    checked, or null otherwise.
     * 5. If the returned value is expected to be a URL (e.g. element is `<a>` and attribute is `href`), returns the
     *    fully resolved URL from the `href`/`src` property of the element, not the attribute.
     * 6. If `name` is 'class', returns the `className` property of the element.
     * 7. If `name` is 'readonly', returns 'true' if the `readOnly` property is true, or null otherwise.
     * 8. If `name` corresponds to a property of the element, and the property is not an Object, return the property
     *    value coerced to a string.
     * 9. If `name` corresponds to an attribute of the element, return the attribute value.
     *
     * @param name The property or attribute name.
     * @returns The value of the attribute as a string, or `null` if no such property or
     * attribute exists.
     */
    getSpecAttribute(name: string): Task<string>;
    /**
     * Gets an attribute of the element.
     *
     * @see [[Element.getProperty]] to retrieve an element property.
     * @param name The name of the attribute.
     * @returns The value of the attribute, or `null` if no such attribute exists.
     */
    getAttribute(name: string): Task<string>;
    /**
     * Gets a property of the element.
     *
     * @see [[Element.getAttribute]] to retrieve an element attribute.
     * @param name The name of the property.
     * @returns The value of the property.
     */
    getProperty(name: string): Task<any>;
    /**
     * Determines if this element is equal to another element.
     */
    equals(other: Element): Task<boolean>;
    /**
     * Returns whether or not the element would be visible to an actual user. This means that the following types
     * of elements are considered to be not displayed:
     *
     * 1. Elements with `display: none`
     * 2. Elements with `visibility: hidden`
     * 3. Elements positioned outside of the viewport that cannot be scrolled into view
     * 4. Elements with `opacity: 0`
     * 5. Elements with no `offsetWidth` or `offsetHeight`
     */
    isDisplayed(): Task<boolean>;
    /**
     * Gets the position of the element relative to the top-left corner of the document, taking into account
     * scrolling and CSS transformations (if they are supported).
     */
    getPosition(): Task<{
        x: number;
        y: number;
    }>;
    /**
     * Gets the size of the element, taking into account CSS transformations (if they are supported).
     */
    getSize(): Task<{
        width: number;
        height: number;
    }>;
    /**
     * Gets a CSS computed property value for the element.
     *
     * @param propertyName
     * The CSS property to retrieve. This argument must be hyphenated, *not* camel-case.
     */
    getComputedStyle(propertyName: string): Task<string>;
    /**
     * Gets the first [[Element.isDisplayed displayed]] element inside this element
     * matching the given query. This is inherently slower than [[Element.find]], so should only be
     * used in cases where the visibility of an element cannot be ensured in advance.
     *
     * @since 1.6
     *
     * @param using
     * The element retrieval strategy to use. See [[Session.find]] for options.
     *
     * @param value
     * The strategy-specific value to search for. See [[Session.find]] for details.
     */
    findDisplayed(using: string, value: string): Task<Element>;
    /**
     * Waits for all elements inside this element that match the given query to be destroyed.
     *
     * @method waitForDeleted
     * @memberOf module:leadfoot/Element#
     *
     * @param using
     * The element retrieval strategy to use. See [[Session.find]] for options.
     *
     * @param value
     * The strategy-specific value to search for. See [[Session.find]] for details.
     */
    waitForDeleted(strategy: string, value: string): Task<void>;
}
export declare type ElementOrElementId = {
    ELEMENT: string;
} | Element | string;
