import Task from '@dojo/core/async/Task';
import { RequestOptions } from '@dojo/core/request';
import { NodeRequestOptions } from '@dojo/core/request/providers/node';
import Session from './Session';
import { Capabilities, LeadfootURL } from './interfaces';
export default class Server {
    url: string;
    requestOptions: RequestOptions;
    /**
     * An alternative session constructor. Defaults to the standard [[Session]] constructor if
     * one is not provided.
     */
    sessionConstructor: typeof Session;
    /**
     * Whether or not to perform capabilities testing and correction when creating a new Server.
     */
    fixSessionCapabilities: boolean;
    /**
     * The Server class represents a remote HTTP server implementing the WebDriver wire protocol that can be used to
     * generate new remote control sessions.
     *
     * @param url
     * The fully qualified URL to the JsonWireProtocol endpoint on the server. The default endpoint for a
     * JsonWireProtocol HTTP server is http://localhost:4444/wd/hub. You may also pass a parsed URL object which will
     * be converted to a string.
     * @param options
     * Additional request options to be used for requests to the server.
     */
    constructor(url: string | LeadfootURL, options?: NodeRequestOptions);
    /**
     * A function that performs an HTTP request to a JsonWireProtocol endpoint and normalises response status and
     * data.
     *
     * @param method
     * The HTTP method to fix
     *
     * @param path
     * The path-part of the JsonWireProtocol URL. May contain placeholders in the form `/\$\d/` that will be
     * replaced by entries in the `pathParts` argument.
     *
     * @param requestData
     * The payload for the request.
     *
     * @param pathParts Optional placeholder values to inject into the path of the URL.
     */
    private _sendRequest<T>(method, path, requestData, pathParts?);
    get<T>(path: string, requestData?: Object, pathParts?: string[]): Task<any>;
    post<T>(path: string, requestData?: Object, pathParts?: string[]): Task<any>;
    delete<T>(path: string, requestData?: Object, pathParts?: string[]): Task<any>;
    /**
     * Gets the status of the remote server.
     *
     * @returns An object containing arbitrary properties describing the status of the remote
     * server.
     */
    getStatus(): Task<any>;
    /**
     * Creates a new remote control session on the remote server.
     *
     * @param desiredCapabilities
     * A hash map of desired capabilities of the remote environment. The server may return an environment that does
     * not match all the desired capabilities if one is not available.
     *
     * @param requiredCapabilities
     * A hash map of required capabilities of the remote environment. The server will not return an environment that
     * does not match all the required capabilities if one is not available.
     */
    createSession<S extends Session>(desiredCapabilities: Capabilities, requiredCapabilities?: Capabilities): Task<S>;
    private _fillCapabilities(session);
    /**
     * Gets a list of all currently active remote control sessions on this server.
     */
    getSessions(): Task<Session[]>;
    /**
     * Gets information on the capabilities of a given session from the server. The list of capabilities returned
     * by this command will not include any of the extra session capabilities detected by Leadfoot and may be
     * inaccurate.
     */
    getSessionCapabilities(sessionId: string): Task<Capabilities>;
    /**
     * Terminates a session on the server.
     */
    deleteSession(sessionId: string): Task<void>;
}
export declare type Method = 'post' | 'get' | 'delete';
