/// <reference types="node" />
import Element from './Element';
import Server from './Server';
import Task from '@dojo/core/async/Task';
import statusCodes from './lib/statusCodes';
import Locator from './lib/Locator';
import { Capabilities, Geolocation, LogEntry, WebDriverCookie } from './interfaces';
export default class Session extends Locator<Task<Element>, Task<Element[]>, Task<void>> {
    private _sessionId;
    private _server;
    private _capabilities;
    private _closedWindows;
    private _timeouts;
    private _movedToElement;
    private _lastMousePosition;
    private _lastAltitude;
    private _nextRequest;
    /**
     * A Session represents a connection to a remote environment that can be driven programmatically.
     *
     * @param sessionId The ID of the session, as provided by the remote.
     * @param server The server that the session belongs to.
     * @param capabilities A map of bugs and features that the remote environment exposes.
     */
    constructor(sessionId: string, server: Server, capabilities: Capabilities);
    /**
     * Information about the available features and bugs in the remote environment.
     *
     * @readonly
     */
    readonly capabilities: Capabilities;
    /**
     * The current session ID.
     *
     * @readonly
     */
    readonly sessionId: string;
    /**
     * The Server that the session runs on.
     *
     * @readonly
     */
    readonly server: Server;
    /**
     * Delegates the HTTP request for a method to the underlying [[Server]] object.
     *
     * @private
     */
    private _delegateToServer<T>(method, path, requestData, pathParts?);
    serverGet<T>(path: string, requestData?: any, pathParts?: string[]): Task<T>;
    serverPost<T>(path: string, requestData?: any, pathParts?: string[]): Task<T>;
    serverDelete<T>(path: string, requestData?: any, pathParts?: string[]): Task<T>;
    /**
     * Gets the current value of a timeout for the session.
     *
     * @param type The type of timeout to retrieve. One of 'script', 'implicit', or 'page load'.
     * @returns The timeout, in milliseconds.
     */
    getTimeout(type: string): Task<number>;
    /**
     * Sets the value of a timeout for the session.
     *
     * @param type
     * The type of timeout to set. One of 'script', 'implicit', or 'page load'.
     *
     * @param ms
     * The length of time to use for the timeout, in milliseconds. A value of 0 will cause operations to time out
     * immediately.
     */
    setTimeout(type: string, ms: number): Task<void>;
    /**
     * Gets the identifier for the window that is currently focused.
     *
     * @returns A window handle identifier that can be used with other window handling functions.
     */
    getCurrentWindowHandle(): Task<string>;
    /**
     * Gets a list of identifiers for all currently open windows.
     */
    getAllWindowHandles(): Task<string[]>;
    /**
     * Gets the URL that is loaded in the focused window/frame.
     */
    getCurrentUrl(): Task<string>;
    /**
     * Navigates the focused window/frame to a new URL.
     */
    get(url: string): Task<void>;
    /**
     * Navigates the focused window/frame forward one page using the browser’s navigation history.
     */
    goForward(): Task<void>;
    /**
     * Navigates the focused window/frame back one page using the browser’s navigation history.
     */
    goBack(): Task<void>;
    /**
     * Reloads the current browser window/frame.
     */
    refresh(): Task<void>;
    /**
     * Executes JavaScript code within the focused window/frame. The code should return a value synchronously.
     *
     * @see [[Session.executeAsync]] to execute code that returns values asynchronously.
     *
     * @param script
     * The code to execute. This function will always be converted to a string, sent to the remote environment, and
     * reassembled as a new anonymous function on the remote end. This means that you cannot access any variables
     * through closure. If your code needs to get data from variables on the local end, they should be passed using
     * `args`.
     *
     * @param args
     * An array of arguments that will be passed to the executed code. Only values that can be serialised to JSON, plus
     * [[Element]] objects, can be specified as arguments.
     *
     * @returns
     * The value returned by the remote code. Only values that can be serialised to JSON, plus DOM elements, can be
     * returned.
     */
    execute<T>(script: Function | string, args?: any[]): Task<T>;
    /**
     * Executes JavaScript code within the focused window/frame. The code must invoke the provided callback in
     * order to signal that it has completed execution.
     *
     * @see [[Session.execute]] to execute code that returns values synchronously.
     * @see [[Session.setExecuteAsyncTimeout]] to set the time until an asynchronous script is
     * considered timed out.
     *
     * @param script
     * The code to execute. This function will always be converted to a string, sent to the remote environment, and
     * reassembled as a new anonymous function on the remote end. This means that you cannot access any variables
     * through closure. If your code needs to get data from variables on the local end, they should be passed using
     * `args`.
     *
     * @param args
     * An array of arguments that will be passed to the executed code. Only values that can be serialised to JSON, plus
     * [[Element]] objects, can be specified as arguments. In addition to these arguments, a
     * callback function will always be passed as the final argument to the function specified in `script`. This
     * callback function must be invoked in order to signal that execution has completed. The return value of the
     * execution, if any, should be passed to this callback function.
     *
     * @returns
     * The value returned by the remote code. Only values that can be serialised to JSON, plus DOM elements, can be
     * returned.
     */
    executeAsync<T>(script: Function | string, args?: any[]): Task<any>;
    /**
     * Gets a screenshot of the focused window and returns it in PNG format.
     *
     * @returns A buffer containing a PNG image.
     */
    takeScreenshot(): Task<Buffer>;
    /**
     * Gets a list of input method editor engines available to the remote environment.
     * As of April 2014, no known remote environments support IME functions.
     */
    getAvailableImeEngines(): Task<string[]>;
    /**
     * Gets the currently active input method editor for the remote environment.
     * As of April 2014, no known remote environments support IME functions.
     */
    getActiveImeEngine(): Task<string>;
    /**
     * Returns whether or not an input method editor is currently active in the remote environment.
     * As of April 2014, no known remote environments support IME functions.
     */
    isImeActivated(): Task<boolean>;
    /**
     * Deactivates any active input method editor in the remote environment.
     * As of April 2014, no known remote environments support IME functions.
     */
    deactivateIme(): Task<void>;
    /**
     * Activates an input method editor in the remote environment.
     * As of April 2014, no known remote environments support IME functions.
     *
     * @param engine The type of IME to activate.
     */
    activateIme(engine: string): Task<void>;
    /**
     * Switches the currently focused frame to a new frame.
     *
     * @param id
     * The frame to switch to. In most environments, a number or string value corresponds to a key in the
     * `window.frames` object of the currently active frame. If `null`, the topmost (default) frame will be used.
     * If an Element is provided, it must correspond to a `<frame>` or `<iframe>` element.
     */
    switchToFrame(id: string | number | Element): Task<void>;
    /**
     * Switches the currently focused window to a new window.
     *
     * @param handle
     * The handle of the window to switch to. In mobile environments and environments based on the W3C WebDriver
     * standard, this should be a handle as returned by [[Session.getAllWindowHandles]].
     *
     * In environments using the JsonWireProtocol, this value corresponds to the `window.name` property of a window.
     */
    switchToWindow(handle: string): Task<void>;
    /**
     * Switches the currently focused frame to the parent of the currently focused frame.
     */
    switchToParentFrame(): Task<void>;
    /**
     * Closes the currently focused window. In most environments, after the window has been closed, it is necessary
     * to explicitly switch to whatever window is now focused.
     */
    closeCurrentWindow(): Task<void>;
    /**
     * Sets the dimensions of a window.
     *
     * @param windowHandle
     * The name of the window to resize. See [[Session.switchToWindow]] to learn about valid
     * window names. Omit this argument to resize the currently focused window.
     *
     * @param width
     * The new width of the window, in CSS pixels.
     *
     * @param height
     * The new height of the window, in CSS pixels.
     */
    setWindowSize(width: number, height: number): Task<void>;
    setWindowSize(windowHandle: string, width: number, height: number): Task<void>;
    /**
     * Gets the dimensions of a window.
     *
     * @param windowHandle
     * The name of the window to query. See [[Session.switchToWindow]] to learn about valid
     * window names. Omit this argument to query the currently focused window.
     *
     * @returns
     * An object describing the width and height of the window, in CSS pixels.
     */
    getWindowSize(windowHandle?: string): Task<{
        width: number;
        height: number;
    }>;
    /**
     * Sets the position of a window.
     *
     * Note that this method is not part of the W3C WebDriver standard.
     *
     * @param windowHandle
     * The name of the window to move. See [[Session.switchToWindow]] to learn about valid
     * window names. Omit this argument to move the currently focused window.
     *
     * @param x
     * The screen x-coordinate to move to, in CSS pixels, relative to the left edge of the primary monitor.
     *
     * @param y
     * The screen y-coordinate to move to, in CSS pixels, relative to the top edge of the primary monitor.
     */
    setWindowPosition(x: number, y: number): Task<void>;
    setWindowPosition(windowHandle: string, x: number, y: number): Task<void>;
    /**
     * Gets the position of a window.
     *
     * Note that this method is not part of the W3C WebDriver standard.
     *
     * @param windowHandle
     * The name of the window to query. See [[Session.switchToWindow]] to learn about valid
     * window names. Omit this argument to query the currently focused window.
     *
     * @returns
     * An object describing the position of the window, in CSS pixels, relative to the top-left corner of the
     * primary monitor. If a secondary monitor exists above or to the left of the primary monitor, these values
     * will be negative.
     */
    getWindowPosition(windowHandle?: string): Task<{
        x: number;
        y: number;
    }>;
    /**
     * Maximises a window according to the platform’s window system behaviour.
     *
     * @param windowHandle
     * The name of the window to resize. See [[Session.switchToWindow]] to learn about valid
     * window names. Omit this argument to resize the currently focused window.
     */
    maximizeWindow(windowHandle?: string): Task<void>;
    /**
     * Gets all cookies set on the current page.
     */
    getCookies(): Task<any[]>;
    /**
     * Sets a cookie on the current page.
     */
    setCookie(cookie: WebDriverCookie): Task<void>;
    /**
     * Clears all cookies for the current page.
     */
    clearCookies(): Task<any>;
    /**
     * Deletes a cookie on the current page.
     *
     * @param name The name of the cookie to delete.
     */
    deleteCookie(name: string): Task<void>;
    /**
     * Gets the HTML loaded in the focused window/frame. This markup is serialised by the remote environment so
     * may not exactly match the HTML provided by the Web server.
     */
    getPageSource(): Task<string>;
    /**
     * Gets the title of the top-level browsing context of the current window or tab.
     */
    getPageTitle(): Task<string>;
    /**
     * Gets the first element from the focused window/frame that matches the given query.
     *
     * @see [[Session.setFindTimeout]] to set the amount of time it the remote environment
     * should spend waiting for an element that does not exist at the time of the `find` call before timing
     * out.
     *
     * @param using
     * The element retrieval strategy to use. One of 'class name', 'css selector', 'id', 'name', 'link text',
     * 'partial link text', 'tag name', 'xpath'.
     *
     * @param value
     * The strategy-specific value to search for. For example, if `using` is 'id', `value` should be the ID of the
     * element to retrieve.
     */
    find(using: string, value: string): Task<Element>;
    /**
     * Gets an array of elements from the focused window/frame that match the given query.
     *
     * @param using
     * The element retrieval strategy to use. See [[Session.find]] for options.
     *
     * @param {string} value
     * The strategy-specific value to search for. See [[Session.find]] for details.
     */
    findAll(using: string, value: string): Task<Element[]>;
    /**
     * Gets the currently focused element from the focused window/frame.
     */
    getActiveElement(): Task<Element>;
    /**
     * Types into the focused window/frame/element.
     *
     * @param keys
     * The text to type in the remote environment. It is possible to type keys that do not have normal character
     * representations (modifier keys, function keys, etc.) as well as keys that have two different representations
     * on a typical US-ASCII keyboard (numpad keys); use the values from [[keys]] to type these
     * special characters. Any modifier keys that are activated by this call will persist until they are
     * deactivated. To deactivate a modifier key, type the same modifier key a second time, or send `\uE000`
     * ('NULL') to deactivate all currently active modifier keys.
     */
    pressKeys(keys: string | string[]): Task<void> | Task<{}>;
    /**
     * Gets the current screen orientation.
     *
     * @returns Either 'portrait' or 'landscape'.
     */
    getOrientation(): Task<string>;
    /**
     * Sets the screen orientation.
     *
     * @param orientation Either 'portrait' or 'landscape'.
     */
    setOrientation(orientation: string): Task<void>;
    /**
     * Gets the text displayed in the currently active alert pop-up.
     */
    getAlertText(): Task<string>;
    /**
     * Types into the currently active prompt pop-up.
     *
     * @param text The text to type into the pop-up’s input box.
     */
    typeInPrompt(text: string | string[]): Task<void>;
    /**
     * Accepts an alert, prompt, or confirmation pop-up. Equivalent to clicking the 'OK' button.
     */
    acceptAlert(): Task<void>;
    /**
     * Dismisses an alert, prompt, or confirmation pop-up. Equivalent to clicking the 'OK' button of an alert pop-up
     * or the 'Cancel' button of a prompt or confirmation pop-up.
     */
    dismissAlert(): Task<void>;
    /**
     * Moves the remote environment’s mouse cursor to the specified element or relative position. If the element is
     * outside of the viewport, the remote driver will attempt to scroll it into view automatically.
     *
     * @param element
     * The element to move the mouse to. If x-offset and y-offset are not specified, the mouse will be moved to the
     * centre of the element.
     *
     * @param xOffset
     * The x-offset of the cursor, maybe in CSS pixels, relative to the left edge of the specified element’s
     * bounding client rectangle. If no element is specified, the offset is relative to the previous position of the
     * mouse, or to the left edge of the page’s root element if the mouse was never moved before.
     *
     * @param yOffset
     * The y-offset of the cursor, maybe in CSS pixels, relative to the top edge of the specified element’s bounding
     * client rectangle. If no element is specified, the offset is relative to the previous position of the mouse,
     * or to the top edge of the page’s root element if the mouse was never moved before.
     */
    moveMouseTo(): Task<void>;
    moveMouseTo(xOffset?: number, yOffset?: number): Task<void>;
    moveMouseTo(element?: Element, xOffset?: number, yOffset?: number): Task<void>;
    /**
     * Clicks a mouse button at the point where the mouse cursor is currently positioned. This method may fail to
     * execute with an error if the mouse has not been moved anywhere since the page was loaded.
     *
     * @param button
     * The button to click. 0 corresponds to the primary mouse button, 1 to the middle mouse button, 2 to the
     * secondary mouse button. Numbers above 2 correspond to any additional buttons a mouse might provide.
     */
    clickMouseButton(button?: number): Task<void>;
    /**
     * Depresses a mouse button without releasing it.
     *
     * @param button The button to press. See [[Session.click]] for available options.
     */
    pressMouseButton(button?: number): Task<void>;
    /**
     * Releases a previously depressed mouse button.
     *
     * @param button The button to press. See [[Session.click]] for available options.
     */
    releaseMouseButton(button?: number): Task<void>;
    /**
     * Double-clicks the primary mouse button.
     */
    doubleClick(): Task<void>;
    /**
     * Taps an element on a touch screen device. If the element is outside of the viewport, the remote driver will
     * attempt to scroll it into view automatically.
     *
     * @param element The element to tap.
     */
    tap(element: Element): Task<void>;
    /**
     * Depresses a new finger at the given point on a touch screen device without releasing it.
     *
     * @param x The screen x-coordinate to press, maybe in device pixels.
     * @param y The screen y-coordinate to press, maybe in device pixels.
     */
    pressFinger(x: number, y: number): Task<void>;
    /**
     * Releases whatever finger exists at the given point on a touch screen device.
     *
     * @param x The screen x-coordinate where a finger is pressed, maybe in device pixels.
     * @param y The screen y-coordinate where a finger is pressed, maybe in device pixels.
     */
    releaseFinger(x: number, y: number): Task<void>;
    /**
     * Moves the last depressed finger to a new point on the touch screen.
     *
     * @param x The screen x-coordinate to move to, maybe in device pixels.
     * @param y The screen y-coordinate to move to, maybe in device pixels.
     */
    moveFinger(x: number, y: number): Task<void>;
    /**
     * Scrolls the currently focused window on a touch screen device.
     *
     * @param element
     * An element to scroll to. The window will be scrolled so the element is as close to the top-left corner of the
     * window as possible.
     *
     * @param xOffset
     * An optional x-offset, relative to the left edge of the element, in CSS pixels. If no element is specified,
     * the offset is relative to the previous scroll position of the window.
     *
     * @param yOffset
     * An optional y-offset, relative to the top edge of the element, in CSS pixels. If no element is specified,
     * the offset is relative to the previous scroll position of the window.
     */
    touchScroll(xOffset: number, yOffset: number): Task<void>;
    touchScroll(element?: Element, xOffset?: number, yOffset?: number): Task<void>;
    /**
     * Performs a double-tap gesture on an element.
     *
     * @param element The element to double-tap.
     */
    doubleTap(element?: Element): Task<void>;
    /**
     * Performs a long-tap gesture on an element.
     *
     * @param element The element to long-tap.
     */
    longTap(element?: Element): Task<void>;
    /**
     * Flicks a finger. Note that this method is currently badly specified and highly dysfunctional and is only
     * provided for the sake of completeness.
     *
     * @param element The element where the flick should start.
     * @param xOffset The x-offset in pixels to flick by.
     * @param yOffset The x-offset in pixels to flick by.
     * @param speed The speed of the flick, in pixels per *second*. Most human flicks are 100–200ms, so
     * this value will be higher than expected.
     */
    flickFinger(element: Element, xOffset: number, yOffset: number, speed?: number): Task<void>;
    flickFinger(xOffset: number, yOffset: number, speed?: number): Task<void>;
    /**
     * Gets the current geographical location of the remote environment.
     *
     * @returns
     * Latitude and longitude are specified using standard WGS84 decimal latitude/longitude. Altitude is specified
     * as meters above the WGS84 ellipsoid. Not all environments support altitude.
     */
    getGeolocation(): Task<Geolocation>;
    /**
     * Sets the geographical location of the remote environment.
     *
     * @param location
     * Latitude and longitude are specified using standard WGS84 decimal latitude/longitude. Altitude is specified
     * as meters above the WGS84 ellipsoid. Not all environments support altitude.
     */
    setGeolocation(location: Geolocation): Task<void>;
    /**
     * Gets all logs from the remote environment of the given type. The logs in the remote environment are cleared
     * once they have been retrieved.
     *
     * @param type
     * The type of log entries to retrieve. Available log types differ between remote environments. Use
     * [[Session.getAvailableLogTypes]] to learn what log types are currently available. Not all
     * environments support all possible log types.
     *
     * @returns
     * An array of log entry objects. Timestamps in log entries are Unix timestamps, in seconds.
     */
    getLogsFor(type: string): Task<string[] | LogEntry[]>;
    /**
     * Gets the types of logs that are currently available for retrieval from the remote environment.
     */
    getAvailableLogTypes(): Task<string[]>;
    /**
     * Gets the current state of the HTML5 application cache for the current page.
     *
     * @returns
     * The cache status. One of 0 (uncached), 1 (cached/idle), 2 (checking), 3 (downloading), 4 (update ready), 5
     * (obsolete).
     */
    getApplicationCacheStatus(): Task<number>;
    /**
     * Terminates the session. No more commands will be accepted by the remote after this point.
     */
    quit(): Task<void>;
    /**
     * Searches a document or element subtree for links with the given normalized text. This method works for 'link text'
     * and 'partial link text' search strategies.
     *
     * Note that this method should be passed to an `execute` call, not called directly.
     *
     * @param using The strategy in use ('link text' or 'partial link text')
     * @param value The link text to search for
     * @param multiple If true, return all matching links
     * @param element A context element
     * @returns The found element or elements
     */
    private _manualFindByLinkText(using, value, multiple, element?);
    /**
     * Gets the list of keys set in local storage for the focused window/frame.
     */
    getLocalStorageKeys(): Task<string[]>;
    /**
     * Sets a value in local storage for the focused window/frame.
     *
     * @param key The key to set.
     * @param value The value to set.
     */
    setLocalStorageItem(key: string, value: string): Task<void>;
    /**
     * Clears all data in local storage for the focused window/frame.
     */
    clearLocalStorage(): Task<void>;
    /**
     * Gets a value from local storage for the focused window/frame.
     *
     * @param key The key of the data to get.
     */
    getLocalStorageItem(key: string): Task<string>;
    /**
     * Deletes a value from local storage for the focused window/frame.
     *
     * @param key The key of the data to delete.
     */
    deleteLocalStorageItem(key: string): Task<void>;
    /**
     * Gets the number of keys set in local storage for the focused window/frame.
     */
    getLocalStorageLength(): Task<number>;
    /**
     * Gets the list of keys set in session storage for the focused window/frame.
     */
    getSessionStorageKeys(): Task<string[]>;
    /**
     * Sets a value in session storage for the focused window/frame.
     *
     * @param key The key to set.
     * @param value The value to set.
     */
    setSessionStorageItem(key: string, value: string): Task<void>;
    /**
     * Clears all data in session storage for the focused window/frame.
     */
    clearSessionStorage(): Task<void>;
    /**
     * Gets a value from session storage for the focused window/frame.
     *
     * @param key The key of the data to get.
     */
    getSessionStorageItem(key: string): Task<string>;
    /**
     * Deletes a value from session storage for the focused window/frame.
     *
     * @param key The key of the data to delete.
     */
    deleteSessionStorageItem(key: string): Task<void>;
    /**
     * Gets the number of keys set in session storage for the focused window/frame.
     */
    getSessionStorageLength(): Task<number>;
    /**
     * Gets the first [[Element.isDisplayed displayed]] element in the currently active window/frame
     * matching the given query. This is inherently slower than [[Session.find]], so should only be
     * used in cases where the visibility of an element cannot be ensured in advance.
     *
     * @since 1.6
     *
     * @param using
     * The element retrieval strategy to use. See [[Session.find]] for options.
     *
     * @param value
     * The strategy-specific value to search for. See [[Session.find]] for details.
     */
    findDisplayed(using: string, value: string): Task<Element | Element[]>;
    /**
     * Waits for all elements in the currently active window/frame to be destroyed.
     *
     * @param using
     * The element retrieval strategy to use. See [[Session.find]] for options.
     *
     * @param value
     * The strategy-specific value to search for. See [[Session.find]] for details.
     */
    waitForDeleted(strategy: string, value: string): Task<void>;
    /**
     * Gets the timeout for [[Session.executeAsync]] calls.
     */
    getExecuteAsyncTimeout(): Task<number>;
    /**
     * Sets the timeout for [[Session.executeAsync]] calls.
     *
     * @param ms The length of the timeout, in milliseconds.
     */
    setExecuteAsyncTimeout(ms: number): Task<void>;
    /**
     * Gets the timeout for [[Session.find]] calls.
     */
    getFindTimeout(): Task<number>;
    /**
     * Sets the timeout for [[Session.find]] calls.
     *
     * @param ms The length of the timeout, in milliseconds.
     */
    setFindTimeout(ms: number): Task<void>;
    /**
     * Gets the timeout for [[Session.get]] calls.
     */
    getPageLoadTimeout(): Task<number>;
    /**
     * Sets the timeout for [[Session.get]] calls.
     *
     * @param ms The length of the timeout, in milliseconds.
     */
    setPageLoadTimeout(ms: number): Task<void>;
}
export interface SessionError extends Error {
    status?: keyof typeof statusCodes;
}
