/**
 * Locator is a class that supports searching for specific element (E), list
 * (L), and void (V) types by various strategies.
 *
 * Note that this class includes JSONWireProtocol strategies. W3C Webdriver only understands 4 strategies:
 *   1. css selector
 *   2. link text
 *   3. partial link text
 *   4. xpath
 */
declare abstract class Locator<E, L, V> {
    abstract find(strategy: Strategies, value: string): E;
    abstract findAll(strategy: Strategies, value: string): L;
    abstract findDisplayed(strategy: Strategies, value: string): E;
    abstract waitForDeleted(strategy: Strategies, value: string): V;
    /**
     * Gets the first element inside this element matching the given CSS class name.
     *
     * @param className The CSS class name to search for.
     */
    findByClassName(className: string): E;
    /**
     * Gets the first element inside this element matching the given CSS selector.
     *
     * @param selector The CSS selector to search for.
     */
    findByCssSelector(selector: string): E;
    /**
     * Gets the first element inside this element matching the given ID.
     *
     * @param id The ID of the element.
     */
    findById(id: string): E;
    /**
     * Gets the first element inside this element matching the given name attribute.
     *
     * @param name The name of the element.
     */
    findByName(name: string): E;
    /**
     * Gets the first element inside this element matching the given case-insensitive link text.
     *
     * @param text The link text of the element.
     */
    findByLinkText(text: string): E;
    /**
     * Gets the first element inside this element partially matching the given case-insensitive link text.
     *
     * @param text The partial link text of the element.
     */
    findByPartialLinkText(text: string): E;
    /**
     * Gets the first element inside this element matching the given HTML tag name.
     *
     * @param tagName The tag name of the element.
     */
    findByTagName(tagName: string): E;
    /**
     * Gets the first element inside this element matching the given XPath selector.
     *
     * @param path The XPath selector to search for.
     */
    findByXpath(path: string): E;
    /**
     * Gets all elements inside this element matching the given CSS class name.
     *
     * @param className The CSS class name to search for.
     */
    findAllByClassName(className: string): L;
    /**
     * Gets all elements inside this element matching the given CSS selector.
     *
     * @param selector The CSS selector to search for.
     */
    findAllByCssSelector(selector: string): L;
    /**
     * Gets all elements inside this element matching the given name attribute.
     *
     * @param name The name of the element.
     */
    findAllByName(name: string): L;
    /**
     * Gets all elements inside this element matching the given case-insensitive link text.
     *
     * @param text The link text of the element.
     */
    findAllByLinkText(text: string): L;
    /**
     * Gets all elements inside this element partially matching the given case-insensitive link text.
     *
     * @param text The partial link text of the element.
     */
    findAllByPartialLinkText(text: string): L;
    /**
     * Gets all elements inside this element matching the given HTML tag name.
     *
     * @param tagName The tag name of the element.
     */
    findAllByTagName(tagName: string): L;
    /**
     * Gets all elements inside this element matching the given XPath selector.
     *
     * @param path The XPath selector to search for.
     */
    findAllByXpath(path: string): L;
    /**
     * Gets the first [[Element.isDisplayed displayed]] element inside this element
     * matching the given CSS class name. This is inherently slower than [[Element.find]], so should
     * only be used in cases where the visibility of an element cannot be ensured in advance.
     *
     * @since 1.6
     * @param className The CSS class name to search for.
     */
    findDisplayedByClassName(className: string): E;
    /**
     * Gets the first [[Element.isDisplayed displayed]] element inside this element
     * matching the given CSS selector. This is inherently slower than [[Element.find]], so should
     * only be used in cases where the visibility of an element cannot be ensured in advance.
     *
     * @since 1.6
     * @param selector The CSS selector to search for.
     */
    findDisplayedByCssSelector(selector: string): E;
    /**
     * Gets the first [[Element.isDisplayed displayed]] element inside this element
     * matching the given ID. This is inherently slower than [[Element.find]], so should
     * only be used in cases where the visibility of an element cannot be ensured in advance.
     *
     * @since 1.6
     * @param id The ID of the element.
     */
    findDisplayedById(id: string): E;
    /**
     * Gets the first [[Element.isDisplayed displayed]] element inside this element
     * matching the given name attribute. This is inherently slower than [[Element.find]], so should
     * only be used in cases where the visibility of an element cannot be ensured in advance.
     *
     * @since 1.6
     * @param name The name of the element.
     */
    findDisplayedByName(name: string): E;
    /**
     * Gets the first [[Element.isDisplayed displayed]] element inside this element
     * matching the given case-insensitive link text. This is inherently slower than [[Element.find]],
     * so should only be used in cases where the visibility of an element cannot be ensured in advance.
     *
     * @since 1.6
     * @param text The link text of the element.
     */
    findDisplayedByLinkText(text: string): E;
    /**
     * Gets the first [[Element.isDisplayed displayed]] element inside this element
     * partially matching the given case-insensitive link text. This is inherently slower than
     * [[Element.find]], so should only be used in cases where the visibility of an element cannot be
     * ensured in advance.
     *
     * @since 1.6
     * @param text The partial link text of the element.
     */
    findDisplayedByPartialLinkText(text: string): E;
    /**
     * Gets the first [[Element.isDisplayed displayed]] element inside this element
     * matching the given HTML tag name. This is inherently slower than [[Element.find]], so should
     * only be used in cases where the visibility of an element cannot be ensured in advance.
     *
     * @since 1.6
     * @param tagName The tag name of the element.
     */
    findDisplayedByTagName(tagName: string): E;
    /**
     * Gets the first [[Element.isDisplayed displayed]] element inside this element
     * matching the given XPath selector. This is inherently slower than [[Element.find]], so should
     * only be used in cases where the visibility of an element cannot be ensured in advance.
     *
     * @since 1.6
     * @param path The XPath selector to search for.
     */
    findDisplayedByXpath(path: string): E;
    /**
     * Waits for all elements inside this element matching the given CSS class name to be destroyed.
     *
     * @param className The CSS class name to search for.
     */
    waitForDeletedByClassName(className: string): V;
    /**
     * Waits for all elements inside this element matching the given CSS selector to be destroyed.
     *
     * @param selector The CSS selector to search for.
     */
    waitForDeletedByCssSelector(selector: string): V;
    /**
     * Waits for all elements inside this element matching the given ID to be destroyed.
     *
     * @param id The ID of the element.
     */
    waitForDeletedById(id: string): V;
    /**
     * Waits for all elements inside this element matching the given name attribute to be destroyed.
     *
     * @param name The name of the element.
     */
    waitForDeletedByName(name: string): V;
    /**
     * Waits for all elements inside this element matching the given case-insensitive link text to be destroyed.
     *
     * @param text The link text of the element.
     */
    waitForDeletedByLinkText(text: string): V;
    /**
     * Waits for all elements inside this element partially matching the given case-insensitive link text to be
     * destroyed.
     *
     * @param text The partial link text of the element.
     */
    waitForDeletedByPartialLinkText(text: string): V;
    /**
     * Waits for all elements inside this element matching the given HTML tag name to be destroyed.
     *
     * @param tagName The tag name of the element.
     */
    waitForDeletedByTagName(tagName: string): V;
    /**
     * Waits for all elements inside this element matching the given XPath selector to be destroyed.
     *
     * @param path The XPath selector to search for.
     */
    waitForDeletedByXpath(path: string): V;
}
export default Locator;
export declare const strategies: {
    'class name': boolean;
    'css selector': boolean;
    'id': boolean;
    'name': boolean;
    'link text': boolean;
    'partial link text': boolean;
    'tag name': boolean;
    'xpath': boolean;
};
export declare type Strategies = keyof typeof strategies;
export declare function toW3cLocator(using: string, value: string): {
    using: string;
    value: string;
};
